/*
 *	Simple mail user interface for KA9Q IP/TCP package.
 *	A.D. Barksdale Garbee II, aka Bdale, N3EUA
 *	Copyright 1986 Bdale Garbee, All Rights Reserved.
 *	Permission granted for non-commercial copying and use, provided
 *	this notice is retained.
 *	Copyright 1987 1988 Dave Trulli NN2Z, All Rights Reserved.
 *	Permission granted for non-commercial copying and use, provided
 *	this notice is retained.
 *
 * revision history:
 *
 *  v3.3 870467 D. Trulli nn2z
 *  v3.2 870314 D. Trulli nn2z
 *  v3.1 870117 D. Trulli nn2z
 *	Add multiple arguments mail commands.
 *	Send to multiple users.
 *	Add status commands;
 *  v3.0 871225 D. Trulli nn2z
 *	Heavily restructured program to use a index array of message.
 *	More compatible with UNIX type mail commands.
 *  v2.7 871214 D. Trulli nn2z
 *	Cleaned up header and message parsing to prevent lock up.
 *	Revised command structure.
 *  v2.6 870826 Bdale
 *	integrate PA0GRI's interface/gateway to the WA7MBL PBBS
 *  v2.5 870713 Bdale
 *      integrate additional patches from PA0GRI, minor cleanup
 *  v2.4 870614 - P. Karn
 *      "smart gateway" function moved to smtp client code in net.exe.
 *	User interface for sending mail reworked to resemble Berkeley Mail
 *  v2.3 870524
 *      Extensive addition/revision by Gerard PA0GRI.  Now supports a healthy
 *      set of real commands.  
 *  v2.1,2.2
 *      exact change history lost.
 *  v2.0 c.870115
 *      First version with command parser, only send and quit work.
 *  v1.0
 *      First attempt.  Send messages only.
 */

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <ctype.h>
#include <string.h>
#include "bm.h"
#include "files.c"
#include "globals.h"
#include <winsock.h>


extern char version[];
static char copyright1[] = 
"Copyright 1987 Bdale Garbee, Copyright 1988 Dave Trulli NN2Z..";
static char copyright2[] =
"Winsock and Windows support added by William Bytheway AA6ED...";

/* comands valid in bm.rc */

struct token rccmds[] = {
	"smtp", SMTP,
	"host", HOST,
	"user", USER,
	"edit", EDIT,
	"fullname", NAME,
	"reply", REPLY,
	"maxlet", MAXLET,
	"mbox", MBOX,
	"record", RECORD,
	"screen", SCREEN,
	"folder", FOLDER,
	"mqueue", MQUEUE,
	NULLCHAR, 0
};

FILE	*mfile = NULLFILE;
char 	*hostname = NULLCHAR;		/* name of this host from rc file */
char	*username = NULLCHAR;		/* name of this user from rc file */
char	*fullname = NULLCHAR;		/* fullname of this user from rc file */
char	*replyto = NULLCHAR;		/* address for reply-to header */
char	*maildir = NULLCHAR;		/* defined mail directory */
char	*mqueue = NULLCHAR;		/* defined mqueue outbound directory */
char	*savebox = NULLCHAR;		/* name of the mbox text file */
char	*record = NULLCHAR;		/* record  outbound mail in this file */
char	*folder = NULLCHAR;		/* directory for saveing read mail */
char	*editor = NULLCHAR;		/* user's favorite text editor */
char	notename[9];			/* name of current notesfile */
char	notefile[SLINELEN];		/* full pathname of mail text file */
char	*mfilename = notefile;	/* pointer to current mbox or mail file -f */
int	current;			/* the current message number */
int	nmsgs;			/* the number of messages in the notesfile */
int	newmsgs;		/* Number of new unread message */
int	change;			/* indicates that the mail file has been
				 * changed in this session */
int	fflag = 0;		/* true if current notesfile is not an mbox */
int	qflag = 0;		/* true if bm is used just to queue files */
unsigned maxlet = NLET+1;	/* max number of messages in mailbox */
int	tty = 0;		/* tells if stdin is a tty */
struct let *mbox;		/* pointer to the array of messages */


char usage[] = "Usage: bm [-u user] [-f file] \r\n  or bm [-s subject] [-q] addr .. addr\r\n";
char badmsg[] = "Invalid Message number %d\r\n";
char nomail[] = "No messages\r\n";
char noaccess[] = "Unable to access %s\r\n";
int MAIL_READWRITE = 0;

void myexit(void)
{
 WSACleanup();
 printf("Exit called...\n");
}


main(argc,argv)
int argc;
char *argv[];
{
	extern int optind;
	extern char *optarg;
	char *subjectline = NULLCHAR;
	long	tmp;
	int	c;
	int	ret;
	char temp[256];
	short portnumber = 23;

/*   AA6ED  *******************************************
#if	defined(__TURBOC__) || defined(MICROSOFT)
	(void) fclose(stdaux);
	(void) fclose(stdprn);
#endif
*/

	loadconfig();
	atexit(myexit);  /* aa6ed says a WSACleanup is OK here */

	if (qflag == 0 && isatty(fileno(stdin))) {
		/* announce ourselves */
		printf(" %s\r\n%s\r\n%s\r\n\r\n",version,copyright1,copyright2);
		tty = 1;
	} 

	current = 1;
	nmsgs = 0;

	/* check for important directories */
	if(access(maildir,0)){
		printf(noaccess,maildir);
		exit(1);
	}
	if(access(mqueue,0)){
		printf(noaccess,mqueue);
		exit(1);
	}
	strncpy(notename,username,8);
	notename[8] = '\0';

	while ((c = getopt(argc,argv,"u:f:s:q")) != -1) {
		switch(c) {
		case 'f':
			fflag++;
			mfilename = optarg;
			break;
		case 'q':
			qflag++;
			break;
		case 's':
			subjectline = optarg;
			break;
		case 'u':
			strncpy(notename,optarg,8);
			notename[8] = '\0';
			break;
		case '?':
			printf(usage);
			exit(1);
		}
	}

	/* set any signal handlers to catch break */
	setsignals();

	if(optind < argc){
		dosmtpsend(0,NULLFILE,&argv[optind],argc-optind,subjectline);
		(void) exit(0);
	}

	tmp = (long)maxlet * (long)sizeof(struct let);
#ifdef	MSDOS
	/*
	* Since we are in the dos small model make sure that we
	* don't overflow a unsigned short on the number bytes
	* need for mallloc. If not checked malloc will
	* succeed and we will be trashing ourself in no time.
	*/
	if ((tmp & 0xffff0000) ||
	    (mbox=(struct let *)malloc((unsigned short)tmp)) == (struct let *)NULL){
#else
	if ((mbox=(struct let *)malloc((unsigned)tmp)) == (struct let *)NULL){
#endif
		fprintf(stderr,
		"Can't allocate memory table for %d messages\r\n",
			 maxlet);
		(void) exit(1);
	}

	printf("Port number 23 used for connects.\n");
	Server_Process(portnumber); /* does command processing */
	return 0;
}

loadconfig(int new_s)
{
	FILE	*rcfp;	/* handle for the configuration file */
	char	rcline[LINELEN]; /* buffer for config file reading */
	register char *s,*p;
	int line = 0;
	char runcom[LINELEN];
	char *getenv();
	char temp[256];

#ifdef UNIX
	/* Try $HOME/RUNCOM */
	if ((p = getenv("HOME")) != NULLCHAR)
		sprintf(runcom, "%s/%s", p, RUNCOM);
#else
	/* check for BMRC in the ENV */
	if ((p = getenv("BMRC")) != NULLCHAR)
		strcpy(runcom,p);
#endif
	else
		strcpy(runcom,RUNCOM);

	if ((rcfp = fopen(runcom,"r")) == NULLFILE) {	/* open config file */
		sprintf(temp,"Cannot open '%s', check your installation\r\n",runcom);
		xprintf(new_s,temp);
		(void) exit(1);
	}
	while (!feof(rcfp)) {
		if (fgets(rcline,LINELEN,rcfp) == NULLCHAR)
			break;
		line++;
		rip(rcline);
		if (*rcline == '\0' || *rcline == ';'|| *rcline == '#')
			continue;
		/* find the argument to the command */

		s = rcline;
		/* skip the white space */
		while(*s == ' ' || *s == '\t')
				s++;
		p = s;
		/* skip the command */
		while(*p && *p != ' ' && *p != '\t')
				p++;
		/* skip the white space */
		while(*p == ' ' || *p == '\t')
				p++;
		if (*s == '\0')
			continue;

		switch (rc_line_type(s)) {
		case HOST:
			hostname = savestr(p);
			break;
		case USER:
			username = savestr(p);
			break;
		case REPLY:
			replyto = savestr(p);
			break;
		case EDIT:
			editor = savestr(p);
			break;
		case SMTP:
			maildir = savestr(p);
			break;
		case NAME:
			fullname = savestr(p);
			break;
		case MAXLET:
			maxlet = atoi(p)+1;
			break;
		case MBOX:
			savebox = savestr(p);
			break;
		case RECORD:
			record = savestr(p);
			break;
		case SCREEN:
#if	defined(__TURBOC__)
			setvideo(p);
#endif
			break;
		case FOLDER:
			folder = savestr(p);
			break;
		case MQUEUE:
			mqueue = savestr(p);
			break;
		default:
			fprintf(stderr,
			"%s: line %d Invalid command: '%s'\r\n",
			runcom,line,rcline);
			exit(1);
		}
	}
	(void) fclose(rcfp);
	if(maildir == NULLCHAR)
		maildir = mailspool;
	if(mqueue == NULLCHAR)
		mqueue = mailqdir;
	if(savebox == NULLCHAR)
		savebox = "mbox";
	if(hostname == NULLCHAR) {
		fprintf(stderr,"%s: hostname not set\r\n",runcom);
		exit(1);
	}
	if(username == NULLCHAR) {
		fprintf(stderr,"%s: username not set\n",runcom);
		exit(1);
	}
}

/* return the line_type from a line of the configuration file */
rc_line_type(s)
register char *s;
{
	register struct token *tp;
	for (tp = rccmds; tp->str != NULLCHAR; tp++) {
		if (strncmp(tp->str,s,strlen(tp->str)) == 0)
			return tp->type;
	}
	return (NONE);
}

/* replace terminating end of line marker(s) with null */
rip(s)
register char *s;
{
	for (; *s; s++)
		if (*s == '\r' || *s == '\n') {
			*s = '\0';
			break;
		}
}

/* copy a string return a pointer to it */
char *
savestr(s)
char *s;
{
	register  char *p;
	p = malloc(strlen(s)+1);
	if (p == NULLCHAR) 
		printf("No more memory\n");
	else
		strcpy(p,s);
	return p;
}

dohelp(int s)
{
 extern int MAIL_READWRITE;

	screen_clear(s);
	xprintf(s,"\r\n\r\n");
	xprintf(s,"------------------------------------------------\r\n");
	xprintf(s," r [msg]  READ message number #\r\n");


	xprintf(s," l        LIST message HEADERS in notefile\r\n");
	xprintf(s," a [name] AREAS change\r\n");
	xprintf(s," .        DISPLAY current message\r\n");
  if (MAIL_READWRITE)
  {
	xprintf(s," s list   SEND a message to list of names\r\n");
	xprintf(s," p [msg]  REPLY to a message #\r\n");
	xprintf(s," f [msg]  FORWARD message #\r\n");
	xprintf(s," e [msg]  RESEND (bounce a message)\r\n");
	xprintf(s," m        MESSAGES in send queue\r\n");
	xprintf(s," a [file] display or change AREA\r\n"); 
	xprintf(s,"-------------------------------------------------\r\n");
	xprintf(s," d [msg # list] DELETE a message # list\r\n");
	xprintf(s," u [msg # list] UNDELETE a message # list\r\n");
	}
/*	xprintf(s,"-------------------------------------------------\r\n");
	xprintf(s," k        KILL unsent messages\r\n");
*/
	xprintf(s,"-------------------------------------------------\r\n");
	xprintf(s," q   QUIT\r\n");
	if (MAIL_READWRITE)
	xprintf(s," b   BYE (update mail that has been read\r\n\r\n");
}


/* save a message list in a file in mailbox format */
savemsg(new_s,argc,argv)
int argc;
register char *argv[];
int new_s;
{
	register char *savefile;
	int msgnum;
	int	i;
	FILE *tfile;
	char buf[LINELEN];
	char temp[256];

	if (mfile == NULLFILE){
		xprintf(new_s,nomail);
		return;
	}
	if (argc == 0 || isdigit(*argv[argc - 1])) {
		savefile = savebox;
	} else {
		savefile = argv[argc - 1];
		--argc;
		/* if it is just a file name and not a path name then check
		** for a folder path defined and use that path to
		** save the file.
		*/
		if (strpbrk(savefile,"/\\") == NULLCHAR && folder != NULLCHAR) {
			sprintf(buf,"%s/%s",folder,savefile);
			savefile = buf;
		}
	}
	if ((tfile = fopen(savefile,"a")) == NULLFILE) {
		perror(savefile);
		return;
	}
	if (argc == 0)
		msgtofile(current, tfile, 0);
	else {
		for (i = 0; i < argc; i++) {
			msgnum = atoi(argv[i]);
			if (msgnum >= 1 && msgnum <= nmsgs)
				msgtofile(msgnum, tfile, 0);
			else
			{
				sprintf(temp,badmsg,msgnum);
				xprintf(new_s,temp);
			}
		}
	}
		(void) fclose(tfile);
		sprintf(temp,"%s appended\r\n",savefile);
		xprintf(new_s,temp);
}

#ifdef MSDOS
/* send messages to the print device */
printmsg(new_s,argc,argv)
int argc;
char *argv[];
int new_s;
{
	FILE *prn;
	int msgnum;
	int	i;
	char temp[256];

	if (mfile == NULLFILE){
		xprintf(new_s,nomail);
		return;
	}
	if ((prn = fopen("PRN","a")) == NULL) {
		perror("PRN");
		return;
	}
	if (argc == 0)
		msgtofile(current, prn, 0);
	else {
		for (i = 0; i < argc; i++) {
			msgnum = atoi(argv[i]);
			if (msgnum >= 1 && msgnum <= nmsgs)
				msgtofile(msgnum, prn, 0);
			else
			{
				sprintf(temp,badmsg,msgnum);
				xprintf(new_s,temp);
			 }
		}
	}
	fclose(prn);
}
#endif

writemsg(new_s,argc,argv)
int argc;
char *argv[];
int new_s;
{
	char *writefile;
	int msgnum;
	int	i;
	FILE *tfile;
	char temp[256];

	if (mfile == NULLFILE){
		xprintf(new_s,nomail);
		return;
	}
	if (argc == 0 || isdigit(*argv[argc - 1])) {
		xprintf(new_s,"no file specified\r\n");
		return;
	} else {
		writefile = argv[argc - 1];
		--argc;
	}
	if ((tfile = fopen(writefile,"a")) == NULLFILE) {
		perror(writefile);
		return;
	}
	if (argc == 0)
		msgtofile(current, tfile, 1);
	else {
		for (i = 0; i < argc; i++) {
			msgnum = atoi(argv[i]);
			if (msgnum >= 1 && msgnum <= nmsgs)
				msgtofile(msgnum, tfile, 1);
			else
			{
				sprintf(temp,badmsg,msgnum);
				xprintf(new_s,temp);
			}
		}
	}
	(void) fclose(tfile);
	sprintf(temp,"%s appended\r\n",writefile);
	xprintf(new_s,temp);
}

bmexit(new_s,x)
int new_s;
int x;
{
	if(!fflag && lockit(new_s))
		exit(1);
	(void) closenotes(new_s);
	if (!fflag)
		rmlock(maildir,notename);
	exit(x);
}

/* this is the main command processing loop */
getcommand(int new_s)
{
	FILE *tfile, *tmpfile();
	char	command[LINELEN];	/* command line */
	char	*args[MAXARGS];
	int	nargs;
	char	*cp;
	register int	msgnum;
	register int	i;
	int ret;
	char * temp;
	extern int MAIL_READWRITE;

	 /* allocate memory for string */
	 if ((temp = (char *) malloc(300)) == NULL)
	 {
		 printf("Not enough memory to allocate buffer\n");
		 exit(1);  /* terminate program if out of memory */
	 }

	xprintf(new_s,"\r\nType ? for help.\r\n");

	/***************** command parsing loop *****************/
	while(new_s != 0) {
		sprintf(temp,"\"%s\"> ",notename);
		xprintf(new_s,temp);
		if (xgets(new_s,command) < 0)
		{
		  new_s = 0;
		  free(temp);
		  return(0);
		}

	  /* don't know how to handle this now     xxxxx
		if (feof(stdin))		 someone hit ^Z!
			strcpy(command,"q");	 treat it like 'q' */

		if(*command == '!') {
		  /* AA6ED if (system(&command[1]))
				perror("system");   */
			continue;
		}
		if (*command) {
			cp = command;
			while (*cp && *cp != ' ')
				cp++;
			nargs = parse(cp,args,MAXARGS);
		}

		if (! MAIL_READWRITE)
		 {
			switch (*command) {
			case 's':    /* send */
			case 'p':    /* reply */
			case 'f':    /* forward */
			case 'e':    /* resend */
			case 'm':    /* send queue */
			case 'd':    /* delete  */
			case 'u':    /* undelete */
			case 'k':    /* kill */
					xprintf(new_s,"You are not authorized to use this command...\r\n");
					*command = '#';   /* dummy command */
					break;
			case 'b':    /* bye  quits with updating */
					*command = 'q';  /* quit without updating */
					break;
		  } /* end switch */
		}   /* end if mail_readonly */

		switch (*command) {
		case '\n':   /* do nothing but eat the LF CR TAB characters */
		case '\r':
		case '\t':
			 xprintf(new_s,"\r\n");
			 break;

		case 's':		/* send msg */
			if (nargs == 0) {
				xprintf(new_s,"To: ");
				xgets(new_s,command);
				nargs = parse(command,args,MAXARGS);
			}
			dosmtpsend(new_s,NULLFILE,args,nargs,NULLCHAR);
			break;

		case 'q':		/* quit */
			(void) fclose(mfile);
		  /* 	closesocket(ss);  */
		  free(temp);
			return 0;
			/* (void) exit(0);
			 NOTREACHED */
			break;

		case 'p':			/* rePly */
			if (nargs == 0)
				msgnum = current;
			else
				msgnum = atoi(args[0]);
			if (msgnum >= 1 && msgnum <= nmsgs)
				reply(msgnum);
			else
			{
				sprintf(temp,badmsg,msgnum);
				xprintf(new_s,temp);
			}
			break;

		case 'f':     /* forward */
			if(nargs == 0)
				msgnum = current;
			else 
				msgnum = atoi(args[0]);
			if (msgnum < 1 || msgnum > nmsgs)
			{
				sprintf(temp,badmsg,msgnum);
				xprintf(new_s,temp);
				break;
			}
			if((tfile = tmpfile()) == NULLFILE)
				xprintf(new_s,"\r\nCannot open temp file\r\n");
			else {
				msgtofile(msgnum,tfile,0);
				fseek(tfile,0L,0);
				xprintf(new_s,"To: ");
				xgets(new_s,command);
				nargs = parse(command,args,MAXARGS);
				dosmtpsend(new_s,tfile,args,nargs,NULLCHAR);
				(void) fclose(tfile);
			}
			break;

		case 'e':		/* bounce a message */
			if(nargs == 0)
				msgnum = current;
			else
				msgnum = atoi(args[0]);
			if (msgnum < 1 || msgnum > nmsgs)
			{
				sprintf(temp,badmsg,msgnum);
				xprintf(new_s,temp);
				break;
			}
			if((tfile = tmpfile()) == NULLFILE)
				xprintf(new_s,"\r\nCannot open temp file\r\n");
			else {
				msgtofile(msgnum,tfile,0);
				fseek(tfile,0L,0);
				xprintf(new_s,"To: ");
				xgets(new_s,command);
				nargs = parse(command,args,MAXARGS);
				bouncemsg(new_s,tfile,args,nargs);
				(void) fclose(tfile);
			}
			break;

		case 'u':    /* undelete */
			if (nargs == 0)
				mbox[current].status &= ~DELETE;
			else
				for (i = 0; i < nargs; i++) {
					msgnum = atoi(args[i]);
					if( msgnum >= 1 && msgnum <= nmsgs)
						mbox[msgnum].status &= ~DELETE;
					else
					{
						sprintf(temp,badmsg,msgnum);
						xprintf(new_s,temp);
					}
				}
			break;

		case 'm':		/* display unsent messages */
			listqueue(new_s);
			break;
/*		case 'k':
			if (nargs == 0)
				xprintf(new_s,"No job id specified\r\n");
			else
				for (i = 0; i < nargs; i++) 
					killjob(args[i]);
			break;
*/
		case 'a': 	/* display or change notefiles */
			mboxnames(new_s,nargs,args);
			break;

		case 'b':		/* bye update mail */
			if (isnewmail(new_s)) {
				xprintf(new_s,"New mail has arrived\r\n");
				reinit(new_s);
			} else
			  {
				/* closesocket(ss); */
				 return 0;
			  }
			 /* 	bmexit(0);
			 NOTREACHED */
			break;

		case 'd':		/* delete a message */
			if (nargs == 0)
				delmsg(current);
			else
				for ( i = 0; i < nargs; i++) {
					msgnum = atoi(args[i]);
					if( msgnum >= 1 && msgnum <= nmsgs)
						delmsg(msgnum);
					else
					{
						sprintf(temp,badmsg,msgnum);
						xprintf(new_s,temp);
					}
				}
			break;

		case 'l':		/* list message headers in notesfile */
			listnotes(new_s);
			break;

		case '\0':	/* a blank line prints next message */
			printnext(new_s);
			break;

		case '?':		/* help */
			dohelp(new_s);
			break;

		case  '.':
			displaymsg(new_s,current);
			break;

		 case 'r':
			if(nargs == 0)
				msgnum = current;
			else
				msgnum = atoi(args[0]);
			if (msgnum < 0 || msgnum > nmsgs)
			{
				sprintf(temp,badmsg,msgnum);
				xprintf(new_s,temp);
			}
			else
			{
				 current = msgnum;
				 displaymsg(new_s,current);
			}
			break;

		default:
			  xprintf(new_s,"Invalid command - ? for help\r\n");
			  break;
		}
	}
}

/* list or change mbox */
mboxnames(new_s,argc,argv)
int argc;
char *argv[];
int new_s;
{
	register char *cp;
	int ret;
	char	line[80];
	char	buf[LINELEN];
	char temp[256];
	int i;
	extern int MAIL_READWRITE;

	if(argc != 0) {
		if(!fflag && lockit(new_s))
			return;
		ret = closenotes(new_s);
		if (!fflag)
			rmlock(maildir,notename);
			if (ret != 0)
				return(1);
			if (strpbrk(argv[0],"/\\") != NULLCHAR) {
				fflag = 1;
				mfilename = argv[0];
			} else {
				fflag = 0;
				mfilename = notefile;
				strncpy(notename,argv[0],8);
				notename[8] = '\0';
				for (i =0; i<strlen(notename);i++)
				{
				 if (notename[i] == '\r') notename[i] = '\0';
				 if (notename[i] == '\n') notename[i] = '\0';
				}
				sprintf(notefile,"%s/%s.txt",maildir,notename);
				printf("areas: [%s]\n",notefile);
			}
			if (!fflag && lockit(new_s)) {
				mfile = NULLFILE;
				xprintf(new_s,"Mail file is busy\r\n");
				return;
			}

			if (strstr(notename,username) == NULL)
				MAIL_READWRITE = FALSE;
			else
				MAIL_READWRITE = TRUE;
				
			ret = initnotes(new_s);
			if (!fflag)
				rmlock(maildir,notename);
			if (ret != 0)
				return(1);
			listnotes(new_s);

		} else {  /* he wants to see what notefiles there are */
			sprintf(buf,"%s/*.txt",maildir,notename);
			filedir(buf,0,line);
			while(line[0] != '\0') {
				cp = strchr(line,'.');
				*cp = '\0';
				/* test for ham radio callsign */
				for (i =1;i < 3 && i < strlen(line);i++)
				{
				 switch (line[i])
				 {
				  case '1':
				  case '2':
				  case '3':
				  case '4':
				  case '5':
				  case '6':
				  case '7':
				  case '9':
				  case '0': i = 10;
				  }   /* end switch */
				 }
				 if (i < 5)
						{
						 sprintf(temp,"notesfile -> %s\r\n",line);
						 xprintf(new_s,temp);
						 }
				filedir(buf,1,line);
			}
		}
}

reinit(new_s)
{
	int ret;
	if (!fflag && lockit(new_s))
		return;
	ret = closenotes();
	if (!fflag)
		rmlock(maildir,notename);
	if (ret != 0)
		exit(1);
	if (!fflag && lockit(new_s)) {
		mfile = NULLFILE;
		xprintf(new_s,"Mail file is busy\r\n");
		return;
	}
	ret = initnotes(new_s);
	if (!fflag)
		rmlock(maildir,notename);
	if (ret != 0)
		return(1);
	listnotes(new_s);
}
