/* -*-C-*-
********************************************************************************
*
* File:         wlytcpip.c
* Description:  TCP INET CLIENT to send lisp expressions to WINTERP lisp-server.
*               (this code derived from examples in HP manual 50952-9000
*               "ARPA Services/300 User's Guide" (pp 4-32 to 4-36)).

********************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <string.h>
#include <mem.h>
#include <sys/types.h>
#include <sys/winsock.h>
#include "tcpip.h" /* defines DEFAULT_INET_SERVICE_PORT, etc */
#include "bm.h"
#include "globals.h"


#define MTU_MAX 1500    /* define maximum MTU message size */


/*******************************************************************************
 * Send <message> to the server via socket <s>
 ******************************************************************************/
 int Server_Send(int s,char *message)
{
 int status;
  if (strlen(message) > 0)
  {
	status =  send(s, message, strlen(message),0);
	 if (status < 0)
	 {
		 fprintf(stderr, "Unable to send(%i) [%s].\n", s,message);
		 return (-1);
	  }
	  else
		return status;
  }
  else
	 printf("ya zapped me a null\n");
	 return status;
}

/*******************************************************************************
 * Receive <message> from the server via socket <s>
 *******************************************************************************/
int Server_Receive(int s, char FAR *message, int maxsize)
{
int len = 0;
int flags = 0;
message[0] = 0;

  len = recv(s, message, maxsize, flags);
  message[len] = 0;
  return len;
}

int flush(int s)
{
  char message[255];
  return (recv(s, message, 255, 0));
}

/*******************************************************************************
 * Establish a connection with server process, returning socket descriptor
 * if successful.
*******************************************************************************/
int Client_Process(char *serverhost, u_short port)
{
  int s;			/* connected socket descriptor */
  struct hostent FAR  *hp;		/* pointer to host info for remote host */
  struct servent FAR  *sp;		/* service information pointer */
  struct sockaddr_in  peeraddr_in; /* for peer socket address */
  WORD wVersionRequested;
  WSADATA wsaData;

  wVersionRequested = 0x0101;

  WSAStartup(wVersionRequested, &wsaData);
  /* clear out address structures */
  memset((char *)&peeraddr_in, 0, sizeof(struct sockaddr_in));
  
  /* Set up the peer address to which we will connect. */
  peeraddr_in.sin_family = AF_INET;

  /* Get the host information for the hostname that the
   * user passed in. If serverhost==NULL, then make a quicker connection
	* by bypassing search in /etc/hosts and using loopback adrs directly.*/
  if (serverhost)
  {
	 if ((hp = gethostbyname(serverhost)) == NULL) {
		fprintf(stderr,"Unable to complete gethostbyname.\n",serverhost);
	WSACleanup();
	return -1;
	 }
	 peeraddr_in.sin_addr.s_addr = ((struct in_addr FAR *)(hp->h_addr))->s_addr;
  }
  else {   /* loopback */
	 peeraddr_in.sin_addr.s_addr = inet_addr(DEFAULT_INET_SERVICE_HOSTADDR);
  }

  if (port == 0) {
    /* get port number for service "widget_interp" */
    if ((sp = getservbyname(DEFAULT_INET_SERVICE_NAME, "tcp")) == NULL) {	
		fprintf(stderr, "Unable to find \"%s\" in getservbyname.\n",
			DEFAULT_INET_SERVICE_NAME);
      WSACleanup();
      return -1;
    }
    peeraddr_in.sin_port = sp->s_port;
  }
  else
    peeraddr_in.sin_port = htons(port);

  /* Create the socket. */
  if ((s = socket(AF_INET, SOCK_STREAM, 0))== -1)
  {
	 fprintf(stderr, "Unable to create Socket().\n");
    WSACleanup();
	 return -1;
  }

  /* Try to connect to the remote server at the address
	* which was just built into peeraddr.  */
  if (connect(s, &peeraddr_in, sizeof(struct sockaddr_in)) == -1)
  {
	 fprintf(stderr, "Unable to connect(%i).\n", s);
	 WSACleanup();
    return -1;
  }

  return(s);
}


/*******************************************************************************
 * Establish a server process
*******************************************************************************/
int Server_Process(u_short port)
{
  static int listen_s, new_s;			/* connected socket descriptor */
  struct hostent FAR  *hp;		/* pointer to host info for remote host */
  struct sockaddr_in peeraddr_in; /* for peer socket address */
  char hostname[256];
  int iLength;
  WORD wVersionRequested;
  WSADATA wsaData;
  int status = 0;
  char temp[10];

  wVersionRequested = 0x0101;

  WSAStartup(wVersionRequested, &wsaData);
  /* clear out address structures */

	if (gethostname(hostname, sizeof(hostname)))
	{
	  fprintf(stderr,"Unable to complete gethostname.\n");
	  printf("Please enter hostname of this computer : ");
	  gets(hostname);
	 }

	if ((hp = gethostbyname(hostname)) == NULL)
	{
	  fprintf(stderr,"Unable to complete gethostbyname[%i].\n",hp);
	  WSACleanup();
	  return -1;
	 }

/* Open a socket. */
  if ((listen_s = socket(AF_INET, SOCK_STREAM, 0))== -1)
  {
	 fprintf(stderr, "Unable to create Socket().\n");
	 WSACleanup();
	 return -1;
  }


  peeraddr_in.sin_port=htons(port);
  peeraddr_in.sin_family=AF_INET;
  peeraddr_in.sin_addr.s_addr = ((struct in_addr FAR *)(hp->h_addr))->s_addr;

  /* Bind to the socket  */
  if (bind(listen_s, &peeraddr_in, sizeof(struct sockaddr_in)) == -1)
  {
	 fprintf(stderr, "Unable to bind(%i) Error is %i.\n", listen_s,WSAGetLastError());
	 WSACleanup();
	 return -1;
  }

  /* printf("Listening for call.\n");  */
  if (listen(listen_s,5) != 0)
  {
	 fprintf(stderr, "Unable to invoke listen, Error = %i.\n",WSAGetLastError());
	 WSACleanup();
	 return -1;
  }

  iLength =  sizeof(struct sockaddr_in);
  status = 0;
  while (status == 0)
  {
	  printf("Listening for call.\n");
	  if ((new_s = accept(listen_s, &peeraddr_in, (int far *)&iLength)) == -1)
	  {
		fprintf(stderr, "Unable to accept(%i), Error = %i.\n",new_s, WSAGetLastError());
		WSACleanup();
		return -1;
	  }
	  if (new_s > 0)
	  {
			printf("Call received on socket %i.\n",new_s);
			/* now call BBS routine  */

		/*	flush(new_s);  */
		 if (login(new_s) == 0)  /* good login */
		 {
			if (get_mail(new_s) == 0)
			{
			xprintf(new_s,"Press Return for list of options.");
			xgets(new_s,temp);
			dohelp(new_s);
			getcommand(new_s);
			xprintf(new_s,"Thanks for calling... Good By...\r\n");
			closesocket(new_s);
			new_s = 0;
			}
			else
			{
			  dohelp(new_s);
			  getcommand(new_s);
			  xprintf(new_s,"Please try again later...Good By\r\n");
			  closesocket(new_s);
			  new_s = 0;
			}  /* end of get_mail */

		 }
		 else
		 {
			closesocket(new_s);
			new_s = 0;
		 }   /* end of login */
	  }
	}
  printf("Status of %i received.\n",status);
  return(0);
}


/******************************************************************************
 * Server_Disconnect -- shutdown the connection for further sends. This causes
 * the server to receive an EOF condition after it has received all data from
 * Server_Send().
******************************************************************************/
void Server_Disconnect(int s)
{
  WSACleanup();
  printf("Socket %i is WSACleanup'ed.\n",s);
}

/******************************************************************************
 *
 ******************************************************************************/
int open_client(char * hostname, u_short port)
{
  int hostname_given = 0;	/* true if hostname was spec'd by user */
  int s;			/* connected socket desriptor */
  char* portstr;
  char* hoststr;

	if (strlen(hostname) > 4)
		 hostname_given = 1;

	/* DEFAULT_INET_PORT_ENVVAR in "../src-server/config.h" */
  if (port == 0)
  {
	 if ((portstr = (char *) getenv(DEFAULT_INET_PORT_ENVVAR)) != NULL)
      port = (u_short) atoi(portstr);
    else
      port = DEFAULT_INET_SERVICE_PORT; /* default port # for winterp server */
  }

  if (hostname_given) /* a remote connection w/ host spec'd on command line */
	 s = Client_Process(hostname, port);
  else
  {    /* DEFAULT_INET_HOSTADDR_ENVVAR in "../src-server/config.h" */
	 if ((hoststr = (char *) getenv(DEFAULT_INET_HOSTADDR_ENVVAR)) != NULL)
		s = Client_Process(hoststr, port); /* use hostname given in environ var */
    else
		s = Client_Process(NULL, port); /* a local connection -- use loopback */
  }

  return s;
}

int login(int new_s)
{
 char loginname[LINELEN];
 char password[LINELEN];
 int tries = 0;
 int i;
 extern int MAIL_READWRITE;

 for (;;)
 {
 xprintf(new_s,"Welcome to the AA6ED Amateur Radio BBS Mailer...\r\n\r\n");
 xprintf(new_s,"Please enter your 'callsign' as your login:...\r\n");
 xprintf(new_s,"\r\nlogin: ");
 if (xgets(new_s, loginname)<0) return -1;
/*  printf("[%s]\n",loginname);   */
 if ((strlen(loginname) > 4) && (isalpha(loginname[0]))) break;
 tries++;
 if (tries > 3)
 {
  xprintf(new_s, "Tries exceeded...\r\n");
  printf("Login failed, last login name was %s\n",login);
  closesocket(new_s);
  return -1;
  }
 }
 for (i=0;i<strlen(loginname);i++)
 {
  if (loginname[i] == '\n') loginname[i] = 0;
  if (loginname[i] == '\r') loginname[i] = 0;
 }
 strncpy(notename,loginname,8);
 xprintf(new_s, "Password: ");
 echo_off(new_s);
 if (xgets(new_s, password) < 0) return -1;
 echo_on(new_s);
  if (strstr(password , "NONE") != NULL)
 {
	 printf("Successful READ ONLY login for %s,%s\n",notename,password);
	 xprintf(new_s, "You only have READ privileges...\r\n");
    strcpy(username,"FAILED");
	 MAIL_READWRITE = FALSE;
 }
 else
 {
	 printf("Successful READ_WRITE login for %s,%s\n",notename,password);
	 xprintf(new_s,"You have full READ/SEND privileges..\r\n");
	 strncpy(username,notename,8);
	 MAIL_READWRITE = TRUE;
  }
 sprintf(loginname,"\r\nHello %s.\r\n",notename);
 xprintf(new_s,loginname);
 return 0;
}

int get_mail(int s)
{
  int ret;

  sprintf(notefile,"%s\\%s.txt",maildir,notename);
  printf("get_mail: %s\n",notefile);  
	 if (!fflag && lockit(s))
		return(1);
	 ret = initnotes(s);
	 if (!fflag)
		rmlock(maildir,notename);
	 if (ret != 0)
		return(1);
	 listnotes(s);

  return 0;
}

int xgetinput(int s, char * message, int rsize)
{
 int size;
 int gsize;
 char * tempstr;
 char * p;

 tempstr = malloc(rsize + 1);
 gsize = rsize - 3;  /* greatest readin size */
 message[0] = 0;
 for (;;)
 {
	 size = xygetinput(s,tempstr,rsize);
	 if (size < 1)
	 {
	  free (tempstr);
	  return (size);
	  }
	 strncat(message,tempstr,size);
	 rsize = rsize - size;
	 if ((p = strchr(message,'\r')) != NULL) break;
	 if ((p = strchr(message,'\n')) != NULL) break;
 }
  p[0]   = '\r';
  p[1]   = '\n';
  p[2]   = '\0';
 free(tempstr);
 /* printf("[%s]\n",message);  */
 return rsize;
}

int xygetinput(int s,char * message, int rsize)
{
  struct timeval time;
  struct timeval *timeptr = &time;
  int MaxFD = 0;
  char temp[100];
  fd_set set;
  int xset;
  int hit;
  int status = 0;
  int size = 0;
  int timeout = 0;


	 timeptr->tv_sec = 1;
	 timeptr->tv_usec = 0;
	 temp[0] = 0;

	while (status ==0)
	{

	 FD_ZERO(&set);
	 FD_SET(s,&set);
	 MaxFD = max(MaxFD, s);

	/* wait for an incomming message */
	 xset = select(MaxFD+1,&set,0,0,timeptr);
	 /* printf("[%i]",xset);  */
	 temp[0] = 0;
	 message[0] = 0;
	 switch (xset)
	 {
		case -1:
			{
				printf("Error detected in select.\n");
				return(-1);
				/* WSACleanup();
				exit(0); */
			}
		case 0:        /* simple timeout, check keyboard */
			{
			  timeout++;
			  if (timeout > 600)
				{
				 xprintf(s,"Timeout exceeded...Bye...\r\n");
				 /* closesocket(new_s);  */
				 return (-1);
				}
			  if (kbhit())
			  {
				 gets(temp);
				 strcat(temp,"\r\n");
				 Server_Send(s,temp);
				 if (strstr(temp,"abort") == temp)
				 {
					WSACleanup();
					exit(0);
				  }
				  else
					 printf("Enter 'abort' for a graceful exit.\n");
				}
				break;
			 }
		default:    /* got a real live one here, maybe */
			{
				size = Server_Receive(s,message,rsize);
				switch (size)
				{
				  case 0: /* socket lost */
				  {
					 printf("Socket %i disconnected...\n",s);
					 return -1;
				  }
				case -1:
				  break;  /* no data to receive, loop and try again */

				default:
				 {
				  if (size > 0)      /* got live data */
					 {
					 /* printf("[%i]%s",size,message); */
						timeout = 0;
						return size;
					 }
				 }   /* default size */
			  }  /* switch  size */
			}    /* default xset */
		}  /* switch xset  */
  } /* end while */
}
